@@ -17,7 +17,7 @@
 
 from __future__ import print_function
 
-from pyjava.api.mlsql import Data
+from pyjava.api.mlsql import PythonContext
 from pyjava.utils import *
 
 # 'resource' is a Unix specific module.
@@ -26,6 +26,7 @@ try:
     import resource
 except ImportError:
     has_resource_module = False
+
 import traceback
 
 from pyjava.serializers import \
@@ -57,6 +58,10 @@ def chain(f, g):
 
 def main(infile, outfile):
     try:
+        try:
+            import ray
+        except ImportError:
+            pass
         # set up memory limits
         memory_limit_mb = int(os.environ.get('PY_EXECUTOR_MEMORY', "-1"))
         if memory_limit_mb > 0 and has_resource_module:
@@ -103,17 +108,17 @@ def main(infile, outfile):
         def process():
             input_data = ser.load_stream(infile)
             code = compile(command, '<string>', 'exec')
-
             if is_interactive:
                 global data_manager
-                data_manager = Data(input_data, conf)
+                global context
+                data_manager = PythonContext(input_data, conf)
+                context = data_manager
                 global globals_namespace
                 exec (code, globals_namespace, globals_namespace)
             else:
-                data_manager = Data(input_data, conf)
-                n_local = {"data_manager": data_manager}
-                g_local = {}
-                exec (code, g_local, n_local)
+                data_manager = PythonContext(input_data, conf)
+                n_local = {"data_manager": data_manager, "context": data_manager}
+                exec (code, n_local, n_local)
             out_iter = data_manager.output()
             try:
                 write_int(SpecialLengths.START_ARROW_STREAM, outfile)
@@ -126,6 +131,7 @@ def main(infile, outfile):
 
     except Exception:
         try:
+            write_int(SpecialLengths.ARROW_STREAM_CRASH, outfile)
             write_int(SpecialLengths.PYTHON_EXCEPTION_THROWN, outfile)
             write_with_length(traceback.format_exc().encode("utf-8"), outfile)
         except IOError:
